#include <stdio.h>

float detect_version(FILE *fin)
{
	fseek(fin, 0xDA0DD, SEEK_SET);
	if (getc(fin) == 0x32)
		return 2.0f;
	return -1;
}

void write_byte(FILE* fin, long pos, int byte)
{
	fseek(fin, pos, SEEK_SET);
	putc(byte, fin);
}

int main(int argc, char *argv[])
{
	if (argc != 3)
	{
		printf("Usage: CM2YEAR CM2.EXE <Year>\n");
		return -1;
	}

	printf("CM2YEAR - Championship Manager 2 - Start Year Changer\n");
	printf("Version 0.2 - By Nick+Co\n\n");

	FILE *fin = fopen(argv[1], "r+b");
	if (fin)
	{
	       // Get Year from arguments
	       int year;
	       sscanf(argv[2], "%d", &year);

	       // Detect CM2 Version
	       float version = detect_version(fin);

	       int real_year;
	       if (version == 2.0f)
	       {
			real_year = 96 + ((year - 1996) % 4);
			write_byte(fin, 0x64b69, real_year);

			int base_year = 1900 + (year - 1996) - (real_year - 96);
			write_byte(fin, 0x37841, (base_year - 0x700));
			write_byte(fin, 0x4F59A, (base_year - 0x700));
			write_byte(fin, 0x54A1C, (base_year - 0x700));
			write_byte(fin, 0x57109, (base_year - 0x700));
			write_byte(fin, 0x57172, (base_year - 0x700));
			write_byte(fin, 0x57580, (base_year - 0x700));
			write_byte(fin, 0x57599, (base_year - 0x700) + 1);

			write_byte(fin, 0x575E6, 0x90);
			write_byte(fin, 0x575E7, 0x90);
			write_byte(fin, 0x575EA, 0x02);
			write_byte(fin, 0x575EB, 0x7c);

			write_byte(fin, 0x57651, 0x83);
			if (year >= 1996)
			{
				write_byte(fin, 0x57652, 0xc2);
				write_byte(fin, 0x57653, (year - 1996) - (real_year - 96));
			}
			else
			{
				write_byte(fin, 0x57652, 0xea);
				write_byte(fin, 0x57653, (1996 - year) - (real_year - 96));
			}
			write_byte(fin, 0x57654, 0x83);
			write_byte(fin, 0x57655, 0xfa);
			write_byte(fin, 0x57656, 0x64);
			write_byte(fin, 0x57657, 0x7c);
			write_byte(fin, 0x57658, 0x03);
			write_byte(fin, 0x57659, 0x83);
			write_byte(fin, 0x5765a, 0xea);
			write_byte(fin, 0x5765b, 0x64);
			write_byte(fin, 0x5765c, 0x66);

			write_byte(fin, 0x576CE, (base_year - 0x700));
			write_byte(fin, 0x5EE2C, (base_year - 0x700));
			write_byte(fin, 0x5F0A4, (base_year - 0x700));
			write_byte(fin, 0x7A346, (base_year - 0x700));
			write_byte(fin, 0x8030E, (base_year - 0x700));

			fclose(fin);
	       }
	       else
	       {
			printf("Cannot detect CM2.exe version!");
			fclose(fin);
			return -1;
	       }
	       printf("Version %.02f detected.\n\nYear changed to %d!\n(underlying year: %d - year shift: +%d)\n", version, year, 1900+real_year, real_year-96);
	}
	else
	{
		printf("Unable to open %s!\n", argv[1]);
		return -1;
	}

	return 0;
}